@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from copy import deepcopy
-from typing import Any, TypeVar
+from typing import Any, TypeVar, TYPE_CHECKING
 from collections.abc import Iterable, Sequence
 
 from django import forms
@@ -13,7 +13,6 @@ from django.forms.models import BaseInlineFormSet
 from django.http.request import HttpRequest
 
 from modeltranslation import settings as mt_settings
-from modeltranslation.fields import TranslationField
 from modeltranslation.translator import translator
 from modeltranslation.utils import (
     build_css_class,
@@ -26,6 +25,11 @@ from modeltranslation.utils import (
 from modeltranslation.widgets import ClearableWidgetWrapper
 from modeltranslation._typing import _ListOrTuple
 
+if TYPE_CHECKING:
+    # We depend here or `django-stubs` internal `_FieldsetSpec`,
+    # in case it changes, change import or define this internally.
+    from django.contrib.admin.options import _FieldsetSpec
+
 _ModelT = TypeVar("_ModelT", bound=Model)
 
 
@@ -40,10 +44,10 @@ class TranslationBaseModelAdmin(BaseModelAdmin[_ModelT]):
 
     def _get_declared_fieldsets(
         self, request: HttpRequest, obj: _ModelT | None = None
-    ) -> _ListOrTuple[tuple[str | None, dict[str, Any]]] | None:
+    ) -> _FieldsetSpec | None:
         # Take custom modelform fields option into account
         if not self.fields and hasattr(self.form, "_meta") and self.form._meta.fields:
-            self.fields = self.form._meta.fields  # type: ignore[assignment]
+            self.fields = self.form._meta.fields  # type: ignore[misc]
         # takes into account non-standard add_fieldsets attribute used by UserAdmin
         fieldsets = (
             self.add_fieldsets
@@ -56,11 +60,18 @@ class TranslationBaseModelAdmin(BaseModelAdmin[_ModelT]):
             return [(None, {"fields": self.replace_orig_field(self.get_fields(request, obj))})]
         return None
 
+    def _patch_fieldsets(self, fieldsets: _FieldsetSpec) -> _FieldsetSpec:
+        fieldsets_new = list(fieldsets)
+        for name, dct in fieldsets:
+            if "fields" in dct:
+                dct["fields"] = self.replace_orig_field(dct["fields"])
+        return fieldsets_new
+
     def formfield_for_dbfield(
         self, db_field: Field, request: HttpRequest, **kwargs: Any
-    ) -> forms.Field:
-        field = super().formfield_for_dbfield(db_field, request, **kwargs)
-        self.patch_translation_field(db_field, field, request, **kwargs)
+    ) -> forms.Field | None:
+        if field := super().formfield_for_dbfield(db_field, request, **kwargs):
+            self.patch_translation_field(db_field, field, request, **kwargs)
         return field
 
     def patch_translation_field(
@@ -80,6 +91,8 @@ class TranslationBaseModelAdmin(BaseModelAdmin[_ModelT]):
             pass
         else:
             orig_formfield = self.formfield_for_dbfield(orig_field, request, **kwargs)
+            if orig_formfield is None:
+                return
             field.widget = deepcopy(orig_formfield.widget)
             attrs = field.widget.attrs
             # if any widget attrs are defined on the form they should be copied
@@ -185,17 +198,6 @@ class TranslationBaseModelAdmin(BaseModelAdmin[_ModelT]):
             option = option_new
         return option  # type: ignore[return-value]
 
-    def _patch_fieldsets(
-        self, fieldsets: _ListOrTuple[tuple[str | None, dict[str, Any]]]
-    ) -> _ListOrTuple[tuple[str | None, dict[str, Any]]]:
-        if fieldsets:
-            fieldsets_new = list(fieldsets)
-            for name, dct in fieldsets:
-                if "fields" in dct:
-                    dct["fields"] = self.replace_orig_field(dct["fields"])
-            fieldsets = fieldsets_new
-        return fieldsets
-
     def _patch_prepopulated_fields(self) -> None:
         def localize(sources: Sequence[str], lang: str) -> tuple[str, ...]:
             "Append lang suffix (if applicable) to field list"
@@ -216,7 +218,7 @@ class TranslationBaseModelAdmin(BaseModelAdmin[_ModelT]):
             else:
                 lang = mt_settings.PREPOPULATE_LANGUAGE or get_language()
                 prepopulated_fields[dest] = localize(sources, lang)
-        self.prepopulated_fields = prepopulated_fields
+        self.prepopulated_fields = prepopulated_fields  # type: ignore[misc]
 
     def _get_form_or_formset(
         self, request: HttpRequest, obj: Model | None, **kwargs: Any
@@ -244,7 +246,7 @@ class TranslationBaseModelAdmin(BaseModelAdmin[_ModelT]):
 
     def _get_fieldsets_pre_form_or_formset(
         self, request: HttpRequest, obj: _ModelT | None = None
-    ) -> _ListOrTuple[tuple[str | None, dict[str, Any]]] | None:
+    ) -> _FieldsetSpec | None:
         """
         Generic get_fieldsets code, shared by
         TranslationAdmin and TranslationInlineModelAdmin.
@@ -262,26 +264,6 @@ class TranslationBaseModelAdmin(BaseModelAdmin[_ModelT]):
         fields = list(base_fields) + list(self.get_readonly_fields(request, obj))
         return [(None, {"fields": self.replace_orig_field(fields)})]
 
-    def get_translation_field_excludes(
-        self, exclude_languages: list[str] | None = None
-    ) -> tuple[TranslationField, ...]:
-        """
-        Returns a tuple of translation field names to exclude based on
-        `exclude_languages` arg.
-        TODO: Currently unused?
-        """
-        if exclude_languages is None:
-            exclude_languages = []
-        excl_languages = []
-        if exclude_languages:
-            excl_languages = exclude_languages
-        exclude = []
-        for orig_fieldname, translation_fields in self.trans_opts.all_fields.items():
-            for tfield in translation_fields:
-                if tfield.language in excl_languages and tfield not in exclude:
-                    exclude.append(tfield)
-        return tuple(exclude)
-
     def get_readonly_fields(
         self, request: HttpRequest, obj: _ModelT | None = None
     ) -> _ListOrTuple[str]:
@@ -310,7 +292,7 @@ class TranslationAdmin(TranslationBaseModelAdmin[_ModelT], admin.ModelAdmin[_Mod
                     translation_fields = get_translation_fields(field)
                     editable_new[index : index + 1] = translation_fields
                     display_new[display_index : display_index + 1] = translation_fields
-            self.list_editable = editable_new
+            self.list_editable = editable_new  # type: ignore[misc]
             self.list_display = display_new
 
     def _group_fieldsets(self, fieldsets: list) -> list:
@@ -326,8 +308,7 @@ class TranslationAdmin(TranslationBaseModelAdmin[_ModelT], admin.ModelAdmin[_Mod
             untranslated_fields = [
                 f.name
                 for f in fields
-                if
-                (
+                if (
                     # Exclude the primary key field
                     f is not self.opts.auto_field
                     # Exclude non-editable fields
@@ -382,9 +363,7 @@ class TranslationAdmin(TranslationBaseModelAdmin[_ModelT], admin.ModelAdmin[_Mod
         kwargs = self._get_form_or_formset(request, obj, **kwargs)
         return super().get_form(request, obj, **kwargs)
 
-    def get_fieldsets(
-        self, request: HttpRequest, obj: _ModelT | None = None
-    ) -> _ListOrTuple[tuple[str | None, dict[str, Any]]]:
+    def get_fieldsets(self, request: HttpRequest, obj: _ModelT | None = None) -> _FieldsetSpec:
         return self._get_fieldsets_pre_form_or_formset(request, obj) or self._group_fieldsets(
             self._get_fieldsets_post_form_or_formset(
                 request, self.get_form(request, obj, fields=None), obj
